require "config"

function createObjectName(baseName, level)
	return baseName.."-"..level
end

local function modifyResistances(resistances, decreaseValue, percentValue)

	local decreaseMod = math.ceil(decreaseValue)
	local percentMod = math.ceil(percentValue)
	
	for _,resistance in pairs(resistances) do
	
		resistance.decrease = resistance.decrease or 0
		resistance.percent = resistance.percent or 0
		
		resistance.decrease = resistance.decrease + decreaseMod
		resistance.percent = resistance.percent + percentMod
		
		if resistance.percent > 100 then
			resistance.percent = 100
		end
	end
end

local function generateIngredients(ingredients, target, level)

	for _, ingredient in pairs(ingredients) do
		ingredient.minLevel = ingredient.minLevel or 1
		ingredient.maxLevel = ingredient.maxLevel or 1000
		
		if level >= ingredient.minLevel and level <= ingredient.maxLevel then
			local actualLevel = level - ingredient.minLevel + 1
			if ingredient[1] and ingredient[2] then
				target.ingredients[#target.ingredients + 1] = {ingredient[1], ingredient[2](actualLevel)}
			else
				target.ingredients[#target.ingredients + 1] = {ingredient.name, ingredient.countFn(actualLevel)}
			end
		end
	end
end

local function generateEntityData(objectName, dataMod, level)

	local objectMod = dataMod.objectMod
	local prototypeCopy = table.deepcopy(data.raw[dataMod.typeName][dataMod.baseName])
	
	prototypeCopy.name = objectName
	prototypeCopy.minable.result = objectName
	
	if dataMod.iconName then
		prototypeCopy.icon = "__fort-mod__/graphics/icons/"..createObjectName(dataMod.iconName, level)..".png"
	end

	if objectMod then
		if objectMod.healthFn then
			prototypeCopy.max_health = objectMod.healthFn(level)
		end
		
		if objectMod.ammoCountFn then
			prototypeCopy.automated_ammo_count = objectMod.ammoCountFn(level)
		end
		
		if objectMod.fastReplaceGroup then
			prototypeCopy.fast_replaceable_group = objectMod.fastReplaceGroup
		end

		if objectMod.rangeFn then
			prototypeCopy.attack_parameters.range = prototypeCopy.attack_parameters.range + objectMod.rangeFn(level)
		end
		
		if objectMod.damageModFn then
			prototypeCopy.attack_parameters.damage_modifier = objectMod.damageModFn(level)
		end
		
		if objectMod.baseResists and objectMod.decreaseResFn and objectMod.percentResFn then
			local resistancesCopy = table.deepcopy(objectMod.baseResists)
			modifyResistances(resistancesCopy, objectMod.decreaseResFn(level), objectMod.percentResFn(level))
			prototypeCopy.resistances = resistancesCopy
		end
	end
	
	data.raw[dataMod.typeName][objectName] = prototypeCopy
end

local function generateItemData(objectName, dataMod, level)

	local itemCopy = table.deepcopy(data.raw.item[dataMod.baseName])
	itemCopy.name = objectName
	itemCopy.place_result = objectName

	if dataMod.iconName then
		itemCopy.icon = "__fort-mod__/graphics/icons/"..createObjectName(dataMod.iconName, level)..".png"
	end
	
	if dataMod.itemMod then
		itemCopy.subgroup = dataMod.itemMod.subgroup
	end
	
	data.raw.item[objectName] = itemCopy
end

local function generateRecipeData(objectName, dataMod, level)

	local recipeCopy = table.deepcopy(data.raw.recipe[dataMod.baseName])
	recipeCopy.name = objectName
	recipeCopy.result = objectName
	recipeCopy.enabled = enableAllRecipes
	
	local recipeMod = dataMod.recipeMod
	
	if recipeMod then
		if recipeMod.energyRequiredFn then
			recipeCopy.energy_required = recipeMod.energyRequiredFn(level)
		end
		
		recipeCopy.ingredients = {}
		
		if recipeMod.usePreviousItem then
			recipeCopy.ingredients = recipeCopy.ingredients or {}
		
			local prevObjectName = dataMod.baseName
			if level > 1 then
				prevObjectName = createObjectName(dataMod.baseName, level - 1)
			end
			
			recipeCopy.ingredients[#recipeCopy.ingredients + 1] = {prevObjectName, recipeMod.previousCountFn(level)}
		end
		
		if recipeMod.useItem and recipeMod.useItemName and recipeMod.useItemLevelFn and recipeMod.useItemCountFn then
			recipeCopy.ingredients = recipeCopy.ingredients or {}
			
			local itemName = createObjectName(recipeMod.useItemName, recipeMod.useItemLevelFn(level))
			recipeCopy.ingredients[#recipeCopy.ingredients + 1] = {itemName, recipeMod.useItemCountFn(level)}
			
		end

		if recipeMod.ingredients then
			generateIngredients(recipeMod.ingredients, recipeCopy, level)
		end
	end
	
	data.raw.recipe[objectName] = recipeCopy
end

function generateTechData(objectName, dataMod, level)
	
	local techMod = dataMod.techMod
	
	if techMod then
		
		local techCopy = table.deepcopy(data.raw.technology[techMod.baseTech])
		local techName = createObjectName(techMod.techName, level)

		if dataMod.iconName then
			techCopy.icon = "__fort-mod__/graphics/icons/"..createObjectName(dataMod.iconName, level)..".png"
		end

		techCopy.name = techName
		techCopy.effects[1].recipe = objectName
		techCopy.order = createObjectName(techMod.order, level)
		
		if level == 1 then
			techCopy.prerequisites = {techMod.baseTech}
		else
			techCopy.prerequisites = {createObjectName(techMod.techName, level - 1)}
		end
		
		if not suppressUpgradeFlag then
			techCopy.upgrade = "true"
		end
		
		if techMod.countFn then
			techCopy.unit.count = techMod.countFn(level)
		end
		
		if techMod.timeFn then
			techCopy.unit.time = techMod.timeFn(level)
		end
		
		if techMod.ingredients then
			techCopy.unit = techCopy.unit or {}
			techCopy.unit.ingredients = {}
			generateIngredients(techMod.ingredients, techCopy.unit, level)
		end
		
		data.raw.technology[techCopy.name] = techCopy
	end

end

function generateData(dataMod)

	for level = 1,dataMod.levelCount do
		local objectName = createObjectName(dataMod.objectName, level)
		
		generateEntityData(objectName, dataMod, level)
		generateItemData(objectName, dataMod, level)
		generateRecipeData(objectName, dataMod, level)
		generateTechData(objectName, dataMod, level)
	end
end
